{% extends "redis_metrics/base.html" %}

{% load redis_metric_tags %}

{% block titletext %}{{ slug }} <small>history</small>{% endblock %}
{% block breadcrumb %}
    <li><a href="{% url 'redis_metric_detail' slug %}">{{ slug }} detail</a></li>
    <li class="active">{{ granularity }} history</li>
{% endblock %}

{% block content %}
    <ul class="nav nav-pills history">
    {% for g in granularities %}
    <li><a href="{% url 'redis_metric_history' slug g %}">{{ g|title }}</a></li>
    {% endfor %}
    </ul>

    {# Menu to control the date from which we display metrics #}
    {% metrics_since slug 5 "history" granularity %}

    {# Include the metrics chart #}
    {% metric_history slug granularity since with_data_table=True %}

{% endblock %}
