{% extends "redis_metrics/base.html" %}
{% load redis_metric_tags %}

{% block titletext %}Gauges{% endblock %}
{% block breadcrumb %}
    <li class="active">Gauges</li>
{% endblock %}

{% block content %}
<div class="row">
{% for slug in gauges %}
    <div class="col-lg-4 col-md-4 col-sm-2">
      {% gauge slug %}
    </div>
{% empty %}
    <p>You don't have any gauges set up yet!</p>
{% endfor %}
</div>
{% endblock %}
