{% load redis_metric_tags %}
{% load static %}


<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="description" content="Metrics, powered by django-redis-metrics">
    <title>Metrics</title>
    <link href="{% static 'redis_metrics/css/bs.min.css' %}" rel="stylesheet">
    <link href="{% static 'redis_metrics/css/bs-theme.min.css' %}" rel="stylesheet">
    <link href="https://maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet">
    {% block extrahead %}{% endblock %}
  </head>
  <body>
    <nav class="navbar navbar-inverse navbar-static-top">
      <div class="container">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar" aria-expanded="false" aria-controls="navbar">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="{% url 'redis_metrics_default' %}">Metrics</a>
        </div>
      </div>
    </nav>

    <div class="container">
      <div class="row">
        <div class="col-md-3">{% block nav %}{% metric_list %}{% endblock %}</div>
        <div class="col-md-9">
          <h1>{% block titletext %}{% endblock %}</h1>

          <ul class="breadcrumb">
          <li><a href="{% url 'redis_metrics_default' %}">Metrics</a></li>
          {% block breadcrumb %}{% endblock %}
          </ul>

          {% block content %}{% endblock %}
        </div>
    </div><!-- /.container -->

    <footer class="footer">
      <div class="container">
        <p class="text-muted">Powered by <a href="https://github.com/bradmontgomery/django-redis-metrics">django-redis-metrics</a>.</p>
      </div>
    </footer>
  </body>
</html>
