{% extends "redis_metrics/base.html" %}

{% load redis_metric_tags %}

{% block titletext %}Aggregate Metrics{% endblock %}
{% block breadcrumb %}
    <li><a href="{% url 'redis_metric_aggregate' %}">Select Metrics</a></li>
    <li class="active">Aggregate Metrics</li>
{% endblock %}

{% block content %}
    {% if category %}<h2>{{ category }}</h2>{% endif %}
    <ul class="nav nav-pills history">
    {% for g in granularities %}
    <li><a href="{% url 'redis_metric_aggregate_history' slugs|join:"+" g %}">{{ g|title }}</a></li>
    {% endfor %}
    </ul>

    {% aggregate_detail slugs True %}

{% endblock %}
