{% extends "redis_metrics/base.html" %}


{% block titletext %}Metrics{% endblock %}
{% block breadcrumb %}
    <li class="active">Metrics List</li>
{% endblock %}

{% block content %}
{% if metrics %}
    <p>Want to see multiple metrics at once?
    Check out the <a href="{% url 'redis_metric_aggregate' %}">Aggregates</a>
    </p>
    <p>Click a metric below to view more data.</p>
    {% for category, slugs in metrics.items %}
        <h2>
            <a href="{% url 'redis_metrics_categorize' category %}"
               title="Edit {{ category}}">{{ category }}</a>
        </h2>
        <ul>
        {% for slug in slugs %}
            <li>
            <a href="{% url 'redis_metric_detail' slug %}">{{ slug }}</a>
            </li>
        {% endfor %}
        </ul>
    {% endfor %}
{% else %}
    <p>You don't have any metrics set up yet!</p>
{% endif %}
{% endblock %}
